@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Utils::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Utils::VERSION   = '1.011_005';
+	$Type::Utils::VERSION   = '1.011_006';
 }
 
 $Type::Utils::VERSION =~ tr/_//d;
@@ -624,12 +624,23 @@ my %is_cache;
 my %is_cache_coderef;
 sub is
 {
+	require Types::TypeTiny;
+	no warnings 'uninitialized';
+	
 	my ($type, $value) = @_;
 	my $caller = caller;
-	my $uniq = blessed($type) ? $type->{uniq} : $type;
-	if (!blessed $type) {
+	
+	my $uniq = Types::TypeTiny::is_TypeTiny($type) ? $type->{uniq} : "$type";
+	
+	if (not Types::TypeTiny::is_TypeTiny $type) {
 		my $orig = $type;
-		$type = $is_cache{$caller}{$uniq} || eval { dwim_type($type, for => $caller) };
+		
+		$type = $is_cache{$caller}{$uniq} || do {
+			Types::TypeTiny::is_StringLike($type)
+				? eval { dwim_type("$type", for => $caller) }
+				: undef;
+		};
+		
 		if (blessed $type) {
 			$is_cache{$caller}{$uniq} ||= $type;
 		}
